package dev.shadowsoffire.apotheosis.ench.table;

import net.minecraft.core.registries.Registries;
import java.util.Collections;
import net.minecraftforge.registries.ForgeRegistries;
import net.minecraft.core.Holder;
import java.util.Collection;
import java.util.ArrayList;
import net.minecraft.tags.TagKey;
import java.util.Optional;
import java.util.List;
import dev.shadowsoffire.placebo.codec.CodecProvider;
import com.mojang.datafixers.kinds.Applicative;
import java.util.function.Function;
import dev.shadowsoffire.placebo.codec.PlaceboCodecs;
import com.mojang.datafixers.kinds.App;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import net.minecraft.network.FriendlyByteBuf;
import com.google.gson.GsonBuilder;
import net.minecraftforge.common.crafting.conditions.ICondition;
import java.util.Comparator;
import dev.shadowsoffire.apotheosis.ench.api.IEnchantingBlock;
import net.minecraft.world.level.LevelReader;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.state.BlockState;
import java.util.Iterator;
import com.mojang.serialization.Codec;
import dev.shadowsoffire.apotheosis.Apotheosis;
import java.util.HashMap;
import dev.shadowsoffire.apotheosis.ench.EnchModule;
import net.minecraft.world.level.block.Block;
import java.util.Map;
import com.google.gson.Gson;
import dev.shadowsoffire.placebo.reload.DynamicRegistry;

public class EnchantingStatRegistry extends DynamicRegistry<BlockStats>
{
    public static final Gson GSON;
    public static final EnchantingStatRegistry INSTANCE;
    private final Map<Block, Stats> statsPerBlock;
    private float absoluteMaxEterna;
    
    protected EnchantingStatRegistry() {
        super(EnchModule.LOGGER, "enchanting_stats", true, false);
        this.statsPerBlock = new HashMap<Block, Stats>();
        this.absoluteMaxEterna = 50.0f;
    }
    
    protected void registerBuiltinCodecs() {
        this.registerDefaultCodec(Apotheosis.loc("enchanting_stats"), (Codec)BlockStats.CODEC);
    }
    
    protected void beginReload() {
        super.beginReload();
        this.statsPerBlock.clear();
    }
    
    protected void onReload() {
        super.onReload();
        for (final BlockStats bStats : this.registry.values()) {
            bStats.blocks.forEach(b -> this.statsPerBlock.put(b, bStats.stats));
        }
        this.computeAbsoluteMaxEterna();
    }
    
    public static float getEterna(final BlockState state, final Level world, final BlockPos pos) {
        final Block block = state.m_60734_();
        if (EnchantingStatRegistry.INSTANCE.statsPerBlock.containsKey(block)) {
            return EnchantingStatRegistry.INSTANCE.statsPerBlock.get(block).eterna;
        }
        return state.getEnchantPowerBonus((LevelReader)world, pos);
    }
    
    public static float getMaxEterna(final BlockState state, final Level world, final BlockPos pos) {
        final Block block = state.m_60734_();
        if (EnchantingStatRegistry.INSTANCE.statsPerBlock.containsKey(block)) {
            return EnchantingStatRegistry.INSTANCE.statsPerBlock.get(block).maxEterna;
        }
        return ((IEnchantingBlock)block).getMaxEnchantingPower(state, (LevelReader)world, pos);
    }
    
    public static float getQuanta(final BlockState state, final Level world, final BlockPos pos) {
        final Block block = state.m_60734_();
        if (EnchantingStatRegistry.INSTANCE.statsPerBlock.containsKey(block)) {
            return EnchantingStatRegistry.INSTANCE.statsPerBlock.get(block).quanta;
        }
        return ((IEnchantingBlock)block).getQuantaBonus(state, (LevelReader)world, pos);
    }
    
    public static float getArcana(final BlockState state, final Level world, final BlockPos pos) {
        final Block block = state.m_60734_();
        if (EnchantingStatRegistry.INSTANCE.statsPerBlock.containsKey(block)) {
            return EnchantingStatRegistry.INSTANCE.statsPerBlock.get(block).arcana;
        }
        return ((IEnchantingBlock)block).getArcanaBonus(state, (LevelReader)world, pos);
    }
    
    public static float getQuantaRectification(final BlockState state, final Level world, final BlockPos pos) {
        final Block block = state.m_60734_();
        if (EnchantingStatRegistry.INSTANCE.statsPerBlock.containsKey(block)) {
            return EnchantingStatRegistry.INSTANCE.statsPerBlock.get(block).rectification;
        }
        return ((IEnchantingBlock)block).getQuantaRectification(state, (LevelReader)world, pos);
    }
    
    public static int getBonusClues(final BlockState state, final Level world, final BlockPos pos) {
        final Block block = state.m_60734_();
        if (EnchantingStatRegistry.INSTANCE.statsPerBlock.containsKey(block)) {
            return EnchantingStatRegistry.INSTANCE.statsPerBlock.get(block).clues;
        }
        return ((IEnchantingBlock)block).getBonusClues(state, (LevelReader)world, pos);
    }
    
    public static float getAbsoluteMaxEterna() {
        return EnchantingStatRegistry.INSTANCE.absoluteMaxEterna;
    }
    
    private void computeAbsoluteMaxEterna() {
        this.absoluteMaxEterna = ((BlockStats)this.registry.values().stream().max(Comparator.comparingDouble(s -> s.stats.maxEterna)).get()).stats.maxEterna;
    }
    
    static /* synthetic */ ICondition.IContext access$000(final EnchantingStatRegistry x0) {
        return x0.getContext();
    }
    
    static {
        GSON = new GsonBuilder().setPrettyPrinting().create();
        INSTANCE = new EnchantingStatRegistry();
    }
    
    record Stats(float maxEterna, float eterna, float quanta, float arcana, float rectification, int clues) {
        public static Codec<Stats> CODEC;
        
        public void write(final FriendlyByteBuf buf) {
            buf.writeFloat(this.maxEterna);
            buf.writeFloat(this.eterna);
            buf.writeFloat(this.quanta);
            buf.writeFloat(this.arcana);
            buf.writeFloat(this.rectification);
            buf.writeByte(this.clues);
        }
        
        public static Stats read(final FriendlyByteBuf buf) {
            return new Stats(buf.readFloat(), buf.readFloat(), buf.readFloat(), buf.readFloat(), buf.readFloat(), buf.readByte());
        }
        
        static {
            Stats.CODEC = (Codec<Stats>)RecordCodecBuilder.create(inst -> inst.group((App)PlaceboCodecs.nullableField((Codec)Codec.FLOAT, "maxEterna", (Object)15.0f).forGetter((Function)Stats::maxEterna), (App)PlaceboCodecs.nullableField((Codec)Codec.FLOAT, "eterna", (Object)0.0f).forGetter((Function)Stats::eterna), (App)PlaceboCodecs.nullableField((Codec)Codec.FLOAT, "quanta", (Object)0.0f).forGetter((Function)Stats::quanta), (App)PlaceboCodecs.nullableField((Codec)Codec.FLOAT, "arcana", (Object)0.0f).forGetter((Function)Stats::arcana), (App)PlaceboCodecs.nullableField((Codec)Codec.FLOAT, "rectification", (Object)0.0f).forGetter((Function)Stats::rectification), (App)PlaceboCodecs.nullableField((Codec)Codec.INT, "clues", (Object)0).forGetter((Function)Stats::clues)).apply((Applicative)inst, Stats::new));
        }
    }
    
    public static class BlockStats implements CodecProvider<BlockStats>
    {
        public static Codec<BlockStats> CODEC;
        public final List<Block> blocks;
        public final Stats stats;
        
        public BlockStats(final List<Block> blocks, final Optional<TagKey<Block>> tag, final Optional<Block> block, final Stats stats) {
            this.blocks = new ArrayList<Block>();
            if (!blocks.isEmpty()) {
                this.blocks.addAll(blocks);
            }
            if (tag.isPresent()) {
                this.blocks.addAll(EnchantingStatRegistry.access$000(EnchantingStatRegistry.INSTANCE).getTag((TagKey)tag.get()).stream().map(Holder::m_203334_).toList());
            }
            if (block.isPresent()) {
                this.blocks.add(block.get());
            }
            this.stats = stats;
        }
        
        public Codec<? extends BlockStats> getCodec() {
            return BlockStats.CODEC;
        }
        
        static {
            BlockStats.CODEC = (Codec<BlockStats>)RecordCodecBuilder.create(inst -> inst.group((App)PlaceboCodecs.nullableField(Codec.list(ForgeRegistries.BLOCKS.getCodec()), "blocks", (Object)Collections.emptyList()).forGetter(bs -> bs.blocks), (App)PlaceboCodecs.nullableField(TagKey.m_203877_(Registries.f_256747_), "tag").forGetter(bs -> Optional.empty()), (App)PlaceboCodecs.nullableField(ForgeRegistries.BLOCKS.getCodec(), "block").forGetter(bs -> Optional.empty()), (App)Stats.CODEC.fieldOf("stats").forGetter(bs -> bs.stats)).apply((Applicative)inst, BlockStats::new));
        }
    }
}
